@@ -46,6 +46,7 @@
 #' in model$coefficients. Use \code{plotSubset(model)} and
 #' \code{plotPercentiles(data, model)} to inspect model
 #' @examples
+#' \dontrun{
 #' # Standard example with sample data
 #' normData <- prepareData()
 #' model <- bestModel(normData)
@@ -54,8 +55,7 @@
 #'
 #' # It is possible to specify the variables explicitly - useful to smuggle
 #' # in variables like sex
-#' preselectedModel <- bestModel(normData, predictors = c("L1", "L3", "L1A3",
-#'                               "A2", "A3"))
+#' preselectedModel <- bestModel(normData, predictors = c("L1", "L3", "L1A3", "A2", "A3"))
 #' print(regressionFunction(preselectedModel))
 #'
 #' # Example for modeling based on continuous age variable and raw variable,
@@ -71,7 +71,7 @@
 #' bmi.sex <- bestModel(bmi.data, raw="bmi", predictors = c("L1", "L2", "L3",
 #'                      "A1", "A2", "A3", "L1A1", "L1A2", "L1A3", "L2A1", "L2A2",
 #'                      "L2A3", "L3A1", "L3A2", "L3A3", "sex"), force.in = c("sex"))
-#'
+#'}
 #' @seealso plotSubset, plotPercentiles, plotPercentileSeries, checkConsistency
 #' @export
 bestModel <- function(data,
@@ -89,7 +89,7 @@ bestModel <- function(data,
 
   if (is.null(k)) {
     k <- attr(data, "k")
-  }else if(k > attr(data, "k")){
+  } else if (k > attr(data, "k")) {
     stop("k parameter exceeds the power degrees in the dataset. Please use computePowers with a higher degree in preparating the data. ")
   }
 
@@ -158,7 +158,7 @@ bestModel <- function(data,
   i <- 1
   rAdj <- results$adjr2[i]
 
-   if (terms > 0 && terms <= length(results$adjr2)) {
+  if (terms > 0 && terms <= length(results$adjr2)) {
     i <- terms
     report <- paste0("User specified solution: ", i, " terms")
   } else {
@@ -168,11 +168,13 @@ bestModel <- function(data,
     }
     if (R2 < results$adjr2[i]) {
       report <- paste0(
-        "Specified R2 falls below the value of the most primitive model. Falling back to model 1.")
+        "Specified R2 falls below the value of the most primitive model. Falling back to model 1."
+      )
     } else if (results$adjr2[length(results$adjr2)] < R2) {
       i <- length(results$adjr2)
       report <- (paste0(
-        "Specified R2 exceeds the R2 of the model with the highest fit. Consider rerunning the analysis with higher k value. Falling back to model ", i))
+        "Specified R2 exceeds the R2 of the model with the highest fit. Consider rerunning the analysis with higher k value. Falling back to model ", i
+      ))
     } else {
       while (rAdj < R2) {
         i <- i + 1
@@ -181,29 +183,20 @@ bestModel <- function(data,
       report <- paste0("Final solution: ", i, " terms")
     }
   }
-  report[2] <- paste0("R-Square Adj. amounts to ", round(results$adjr2[i], digits = 6))
-  text <- paste0(raw, " ~ ")
-  names <- colnames(results$outmat)
+  report[2] <- paste0("R-Square Adj. = ", round(results$adjr2[i], digits = 6))
 
-  j <- 1
-  nr <- 0
-  while (j <= length(names)) {
-    if (results$outmat[i, j] == "*") {
-      text1 <- names[j]
-      if (nr == 0) {
-        text <- paste(text, text1, sep = "")
-      } else {
-        text <- paste(text, text1, sep = " + ")
-      }
-
-      nr <- nr + 1
-    }
-    j <- j + 1
-  }
+  variables <- names(coef(subsets, id = i))
+  variables <- variables[2:length(variables)] # remove '(Intercept)' variable
+  text <- paste0(raw, " ~ ", paste(variables, collapse = " + ")) # build regression formula
 
   report[3] <- paste0("Final regression model: ", text)
   bestformula <- lm(text, as.data.frame(data))
 
+  # compute rmse
+  tab <- data.frame(raw = data[, raw], fitted = bestformula$fitted.values)
+  tab <- tab[complete.cases(tab), ]
+  rmse <- sqrt(sum((tab$raw - tab$fitted)^2) / length(tab$raw))
+
   # Model information
   bestformula$ideal.model <- i
   bestformula$cutoff <- R2
@@ -217,6 +210,7 @@ bestModel <- function(data,
   bestformula$minRaw <- min(data[, raw])
   bestformula$maxRaw <- max(data[, raw])
   bestformula$raw <- raw
+  bestformula$rmse <- rmse
   bestformula$scaleSD <- attributes(data)$scaleSD
   bestformula$scaleM <- attributes(data)$scaleM
   bestformula$descend <- attributes(data)$descend
@@ -226,9 +220,10 @@ bestModel <- function(data,
 
   # Print output
   report[4] <- paste0("Regression function: ", regressionFunction(bestformula, digits = 10))
-  report[5] <- ""
+  report[5] <- paste0("RMSE = ", round(rmse, digits = 5))
+
   bestformula$report <- report
-  cat(report, sep='\n')
+  cat(report, sep = "\n")
 
   message("Use 'printSubset(model)' to get detailed information on the different solutions, 'plotSubset(model)' to inspect model fit and 'summary(model)' for statistics on the regression model.")
   return(bestformula)
@@ -253,8 +248,9 @@ printSubset <- function(model) {
   table <-
     do.call(rbind, Map(data.frame,
       R2 = model$subsets$rsq,
-      RSS = model$subsets$rss,
       R2adj = model$subsets$adjr2,
+      RSS = model$subsets$rss,
+      RMSE = sqrt(model$subsets$rss / length(model$fitted.values)),
       Cp = model$subsets$cp,
       BIC = model$subsets$bic
     ))
@@ -294,8 +290,6 @@ printSubset <- function(model) {
 #' @param stepNorm Stepping parameter for the norm table check within age with lower
 #' scores indicating a higher precision. The choice depends of the norm scale
 #' used. With T scores a stepping parameter of 1 is suitable
-#' @param descend Reverse raw scores order. If set to TRUE, lower raw scores
-#' indicate higher performance. Relevant f. e. in case of modeling errors
 #' @param warn If set to TRUE, already minor violations of the model assumptions
 #' are displayed (default = FALSE)
 #' @param silent turn off messages
@@ -316,7 +310,6 @@ checkConsistency <- function(model,
                              maxRaw = NULL,
                              stepAge = 1,
                              stepNorm = 1,
-                             descend = FALSE,
                              warn = FALSE,
                              silent = FALSE) {
   if (is.null(minAge)) {
@@ -342,20 +335,14 @@ checkConsistency <- function(model,
   if (is.null(maxRaw)) {
     maxRaw <- model$maxRaw
   }
-
-  if (is.null(descend)) {
-    descend <- model$descend
-  }
+  descend <- model$descend
 
   i <- minAge
   minor <- 0
   major <- 0
   results <- c()
   while (i <= maxAge) {
-    norm <- normTable(i, model,
-      minNorm = minNorm, maxNorm = maxNorm, minRaw = minRaw, maxRaw = maxRaw,
-      step = stepNorm, descend = descend
-    )
+    norm <- normTable(i, model, minNorm = minNorm, maxNorm = maxNorm, minRaw = minRaw, maxRaw = maxRaw, step = stepNorm)
     k <- 1
     maxR <- 0
     while (k < length(norm$raw)) {
@@ -476,54 +463,59 @@ regressionFunction <- function(model, raw = NULL, digits = NULL) {
   return(formulA)
 }
 
-#' First order derivative of regression model
+#' Derivative of regression model
 #'
-#' Calculates the first order derivative of the location / norm value from the regression model. This
-#' is useful for finding violations of model assumptions and problematic distribution features as
-#' f. e. bottom and ceiling effects, non-progressive norm scores within an age group or in general
-#' intersecting percentile curves.
+#' Calculates the derivative of the location / norm value from the regression model with the first
+#' derivative as the default. This is useful for finding violations of model assumptions and problematic
+#' distribution features as f. e. bottom and ceiling effects, non-progressive norm scores within an
+#' age group or in general #' intersecting percentile curves.
 #' @param model The regression model
+#' @param order The degree of the derivate, default: 1
 #' @return The derived coefficients
 #' @examples
 #' normData <- prepareData()
 #' m <- bestModel(normData)
 #' derivedCoefficients <- derive(m)
 #' @export
-derive <- function(model) {
+derive <- function(model, order = 1) {
   coeff <- model$coefficients[grep("L", names(model$coefficients))]
-  i <- 1
-  name <- names(coeff)
 
 
-  # easy, straight forward derivation of betas and variable names
-  while (i <= length(coeff)) {
-    nam <- strsplit(name[[i]], "")
-
-    if (nam[[1]][1] == "L") {
-      coeff[[i]][1] <- coeff[[i]][1] * as.numeric(nam[[1]][2])
+  for (o in 1:order) {
+    if (o > 1) {
+      coeff <- coeff[grep("L", names(coeff))]
     }
-    nam[[1]][2] <- as.numeric(nam[[1]][2]) - 1
+    i <- 1
+    name <- names(coeff)
+    # easy, straight forward derivation of betas and variable names
+    while (i <= length(coeff)) {
+      nam <- strsplit(name[[i]], "")
 
-    newString <- ""
-
-    if (nchar(name[[i]]) == 2) {
-      if (nam[[1]][2] > 0) {
-        newString <- paste0(nam[[1]][1], nam[[1]][2])
+      if (nam[[1]][1] == "L") {
+        coeff[[i]][1] <- coeff[[i]][1] * as.numeric(nam[[1]][2])
       }
-    } else {
-      if (nam[[1]][2] > 0) {
-        newString <- paste0(nam[[1]][1], nam[[1]][2], nam[[1]][3], nam[[1]][4])
+      nam[[1]][2] <- as.numeric(nam[[1]][2]) - 1
+
+      newString <- ""
+
+      if (nchar(name[[i]]) == 2) {
+        if (nam[[1]][2] > 0) {
+          newString <- paste0(nam[[1]][1], nam[[1]][2])
+        }
       } else {
-        newString <- paste0(nam[[1]][3], nam[[1]][4])
+        if (nam[[1]][2] > 0) {
+          newString <- paste0(nam[[1]][1], nam[[1]][2], nam[[1]][3], nam[[1]][4])
+        } else {
+          newString <- paste0(nam[[1]][3], nam[[1]][4])
+        }
       }
+      name[[i]] <- newString
+
+      i <- i + 1
     }
-    name[[i]] <- newString
 
-    i <- i + 1
+    names(coeff) <- name
   }
-
-  names(coeff) <- name
-
   return(coeff)
 }
 
@@ -557,3 +549,264 @@ rangeCheck <- function(model, minAge = NULL, maxAge = NULL, minNorm = NULL, maxN
 
   return(summary)
 }
+
+#' Cross validation for term selection
+#'
+#' This function helps in selecting the number of terms for the model by doing repeated
+#' cross validation with 80 percent of the data as training data and 20 percent as the validation data.
+#' The cases are drawn randomly but stratified by norm group. Successive models are retrieved
+#' with increasing number of terms and the RMSE of raw scores (fitted by the regression model)
+#' is plotted for the training, validation and the complete dataset. Additionally to this
+#' analysis on the raw score level, it is possible (default) to estimate the mean norm score
+#' reliability and crossfit measures. For this, please set the norms parameter to TRUE. Due
+#' to the high computational load when computing norm scores, it takes time to finish
+#' when doing repeated cv or comparing models up to the maximum number of terms. When using
+#' the cv = "full" option, the ranking is done for the test and validation dataset
+#' separately (always based on T scores), resulting in a complete cross validation. In
+#' order to only validate the modeling, you as well can use a pre ranked data set with
+#' prepareData() already applied. In this case, the training and validation data is
+#' drawn from the already ranked data and the scores for the validation set should improve.
+#' It is however no independent test, as the ranking between both samples is interlinked.
+#'
+#' As a suggestion for real tests, combine visual inspection of the percentiles with a
+#' repeated cross validation (e. g. 10 repetitions). Fokus on norm score R2 in the
+#' validation dataset and avoid models with a high overfit (e. g. crossfit > 1.25).
+#'
+#' @param data data frame of norm sample with ranking, powers and interaction of L and A
+#' @param repetitions number of repetitions for cross validation
+#' @param norms determine norm score crossfit and R2 (if set to TRUE). The option is
+#' computationally intensive and duration increases with sample size, number of
+#' repetitions and maximum number of terms (max option).
+#' @param min Minimum number of terms to start from, default = 1
+#' @param max Maximum number of terms in model up to 2*k + k^2
+#' @param cv If set to full (default), the data is split into training and validation data and ranked afterwards,
+#' otherwise, a pre ranked dataset has to be provided, which is then split into train and validation (and thus
+#' only the modelling, but not the ranking is independent)
+#' @param pCutoff The function checks the stratification for unbalanced data sampling.
+#' It performs a t-test per group . pCutoff specifies the p-value per group that the test result
+#' has to reach at least. To minimize beta error, the value is set to .2 per default
+#' @param width If provided, ranking is done via rankBySlidingWindow, otherwise by group
+#' @param raw Name of the raw variable
+#' @param age Name of the age variable
+#' @param group Name of the grouping variable
+#' @return table with results per term number, including RMSE for raw scores in training, validationand complete
+#' sample, R2 for the norm scores and the crossfit measure (1 = ideal, <1 = underfit, >1 = overfit)
+#' @export
+#' @examples
+#' # plot cross validation RMSE by number of terms up to 9 with three repetitions
+#' data <- prepareData()
+#' cnorm.cv(data, 3, max=7, norms=FALSE)
+cnorm.cv <- function(data, repetitions = 1, norms = TRUE, min = 1, max = 12, cv = "full", pCutoff = .2, width = NA, raw = NA, group = NA, age = NA) {
+
+  d <- data
+
+  if(is.na(raw)||is.na(group)||is.na(age)){
+      raw <- attr(d, "raw")
+      age <- attr(d, "age")
+      group <- attr(d, "group")
+  }
+
+  if(is.na(raw)||is.na(group)||is.na(age)){
+    stop("Variables raw, age and group neither available as function parameters nor as attributes from data object. Please provide according information.")
+  }
+
+  scaleM <- attr(d, "scaleMean")
+  if(is.na(scaleM) || cv=="full"){
+    scaleM <- 50
+  }
+
+  scaleSD <- attr(d, "scaleSD")
+  if(is.na(scaleSD) || cv=="full"){
+    scaleSD <- 10
+  }
+
+  k <- attr(d, "k")
+  if(is.na(k)){
+    k <- 4
+  }
+
+  n.models <- 2 * k + k * k
+  if(is.na(max) || max > n.models || max < 1){
+    max <- n.models
+  }
+
+  # set up regression formulas (from bestModel function)
+  if (k == 1) {
+    lmX <- formula(paste(raw, "L1 + A1 + L1A1", sep = " ~ "))
+  } else if (k == 2) {
+    lmX <-
+      formula(paste(raw, "L1 + L2 + A1 + A2 + L1A1 + L1A2 + L2A1 + L2A2", sep = " ~ "))
+  } else if (k == 3) {
+    lmX <- formula(paste(raw, "L1 + L2 + L3 + A1 + A2 + A3 + L1A1 + L1A2 + L1A3 + L2A1 + L2A2 + L2A3 + L3A1 + L3A2 + L3A3", sep = " ~ "))
+  } else if (k == 4) {
+    lmX <- formula(paste(raw, "L1 + L2 + L3 + L4 + A1 + A2 + A3 + A4 + L1A1 + L1A2 + L1A3 + L1A4 + L2A1 + L2A2 + L2A3 + L2A4 + L3A1 + L3A2 + L3A3 + L3A4 + L4A1 + L4A2 + L4A3 + L4A4", sep = " ~ "))
+  } else if (k == 5) {
+    lmX <- formula(paste(raw, "L1 + L2 + L3 + L4 + L5 + A1 + A2 + A3 + A4 + A5 + L1A1 + L1A2 + L1A3 + L1A4 + L1A5 + L2A1 + L2A2 + L2A3 + L2A4 + L2A5 + L3A1 + L3A2 + L3A3 + L3A4 + L3A5 + L4A1 + L4A2 + L4A3 + L4A4 + L4A5 + L5A1 + L5A2 + L5A3 + L5A4 + L5A5", sep = " ~ "))
+  } else if (k == 6) {
+    lmX <-
+      formula(paste(raw, "L1 + L2 + L3 + L4 + L5 + L6 + A1 + A2 + A3 + A4 + A5 + A6 + L1A1 + L1A2 + L1A3 + L1A4 + L1A5 + L1A6 + L2A1 + L2A2 + L2A3 + L2A4 + L2A5 + L2A6 + L3A1 + L3A2 + L3A3 + L3A4 + L3A5 + L3A6 + L4A1 + L4A2 + L4A3 + L4A4 + L4A5 + L4A6 + L5A1 + L5A2 + L5A3 + L5A4 + L5A5 + L5A6 + L6A1 + L6A2 + L6A3 + L6A4 + L6A5 + L6A6", sep = " ~ "))
+  }
+
+
+  # set up vectors to store RMSE for training, test and complete dataset models
+  val.errors <- rep(0, max)
+  train.errors <- rep(0, max)
+  complete.errors <- rep(0, max)
+
+  # set up vectors to store norm score R2 and CROSSFIT
+  r2.train <- rep(0, max)
+  r2.test <- rep(0, max)
+  delta <- rep(NA, max)
+  crossfit <- rep(0, max)
+
+  # draw test and training data several times ('repetitions' parameter), model data and store MSE
+  for (a in 1:repetitions) {
+
+    # check for imbalances in data and repeat if stratification was unsatisfactory - usually never occurs
+    p.value <- .01
+    n <- 1  # to avoid a deadlock, define stop criterion
+
+    while (p.value < pCutoff) {
+      if(n > 100){
+        stop("Could not establish balanced data sets. Try to decrease pCutoff parameter.")
+      }
+      n <- n + 1
+
+      # shuffle data and split into groups (for stratification)
+      d <- d[sample(nrow(d)), ]
+      d <- d[order(d[, group]), ]
+      sp <- split(d, list(d[, group]))
+      sp <- lapply(sp, function(x) x[sample(nrow(x)), ])
+
+      # draw 9 tenth of data from each group for training and testing
+      train <- lapply(sp, function(x) x[c(FALSE, rep(TRUE, 4)), ])
+      test <- lapply(sp, function(x) x[c(TRUE, rep(FALSE, 4)), ])
+
+      # test for significant differences to avoid extremely unbalanced data
+      p <- rep(1, length(train))
+      for(z in 1:length(train)){
+        p[z] <- t.test(train[[z]][, raw], test[[z]][, raw])$p.value
+      }
+      p.value <- min(p)
+      if(p.value < pCutoff){
+        next
+      }
+
+      # combine lists to data frames
+      train <- do.call(rbind, train)
+      test <- do.call(rbind, test)
+
+      if(cv=="full"){
+        train <- prepareData(train, raw=raw, group=group, age=age)
+        test <- prepareData(test, raw=raw, group=group, age=age)
+      }
+      # test for overall significant differences between groups, restart stratification if necessary
+      #p.value <- t.test(train[, raw], test[, raw])$p.value
+    }
+
+    # compute leaps model
+    subsets <- leaps::regsubsets(lmX, data = train, nbest = 1, nvmax = max, really.big = n.models > 25)
+
+    # retrieve models coefficients for each number of terms
+    for (i in min:max) {
+      cat(paste0("Repetition " , a, ", cycle ", i, "\n"))
+      variables <- names(coef(subsets, id = i))
+      variables <- variables[2:length(variables)] # remove '(Intercept)' variable
+      reg <- paste0(raw, " ~ ", paste(variables, collapse = " + ")) # build regression formula
+
+      # run linear regression for specific model
+      model <- lm(reg, train)
+      model$k <- k
+      model$minRaw <- min(train[, raw])
+      model$maxRaw <- max(train[, raw])
+      model$scaleM <- scaleM
+      model$scaleSD <- scaleSD
+
+
+      # predict values in test data
+      test.fitted <- predict.lm(model, test)
+
+      # store MSE for test and train data
+      train.errors[i] <- train.errors[i] + mean((model$fitted.values - train[, raw])^2)
+      val.errors[i] <- val.errors[i] + mean((test.fitted - test[, raw])^2)
+
+      # compute R2 for test and training
+      if(norms){
+        train$T <- predictNorm(train[, raw], train[, age], model, min(train$normValue), max(train$normValue))
+        test$T <- predictNorm(test[, raw], test[, age], model, min(train$normValue), max(train$normValue))
+
+        r2.train[i] <- r2.train[i] + (cor(train$normValue, train$T, use = "pairwise.complete.obs")^2)
+        r2.test[i] <- r2.test[i] + (cor(test$normValue, test$T, use = "pairwise.complete.obs")^2)
+      }
+    }
+  }
+
+  # now for the complete data the same logic
+  complete <- leaps::regsubsets(lmX, data = d, nbest = 1, nvmax = n.models, really.big = n.models > 25)
+  for (i in 1:max) {
+    variables <- names(coef(complete, id = i))
+    variables <- variables[2:length(variables)]
+    reg <- paste0(raw, " ~ ", paste(variables, collapse = " + "))
+    model <- lm(reg, d)
+
+    # mse for the complete data based on number of terms
+    complete.errors[i] <- sqrt(mean((model$fitted.values - d[, raw])^2))
+
+    # build the average over repetitions and the root
+    train.errors[i] <- sqrt(train.errors[i] / repetitions)
+    val.errors[i] <- sqrt(val.errors[i] / repetitions)
+
+    if(norms){
+      r2.train[i] <- r2.train[i] / repetitions
+      r2.test[i] <- r2.test[i] / repetitions
+
+      if(i > min){
+        delta[i] <- r2.test[i] - r2.test[i - 1]
+      }
+    }
+
+    if(i < min){
+      r2.train[i] <- NA
+      r2.test[i] <- NA
+      val.errors[i] <- NA
+      train.errors[i] <- NA
+      complete.errors[i] <- NA
+    }
+  }
+
+  if(norms){
+    par(mfrow = c(2, 2)) # set the plotting area into a 1*2 array
+  }else{
+    par(mfrow = c(1, 1))
+  }
+  tab <- data.frame(RMSE.raw.train = train.errors, RMSE.raw.test = val.errors, RMSE.raw.complete = complete.errors, r2.train = r2.train, r2.test = r2.test, delta.r2.test = delta, crossfit = r2.train / r2.test)
+
+  # plot RMSE
+  plot(val.errors, pch = 19, type = "b", col = "blue", main = "Raw Score RMSE", ylab = "Root MSE", xlab = "Number of terms", ylim=c(min(train.errors, na.rm = TRUE),max(val.errors, na.rm = TRUE)))
+  points(complete.errors, pch = 19, type = "b", col = "black")
+  points(train.errors, pch = 19, type = "b", col = "red")
+  legend("topright", legend = c("Training", "Validation", "Complete"), col = c("red", "blue", "black"), pch = 19)
+
+  if(norms){
+    # plot R2
+    plot(r2.train, pch = 19, type = "b", col = "red", main = "Norm Score R2", ylab = "R Square", xlab = "Number of terms", ylim=c(min(r2.test, na.rm = TRUE),1))
+    points(r2.test, pch = 19, type = "b", col = "blue")
+    legend("bottomright", legend = c("Training", "Validation"), col = c("red", "blue"), pch = 19)
+
+    # plot CROSSFIT
+    plot(tab$crossfit, pch = 19, type = "b", col = "black", main = "Norm Score CROSSFIT", ylab = "Crossfit", xlab = "Number of terms", ylim=c(min(tab$crossfit, na.rm = TRUE),max(tab$crossfit, na.rm = TRUE)))
+    abline(h = 1, col = 3, lty = 2)
+
+    # plot delta r2 test
+    plot(tab$delta.r2.test, pch = 19, type = "b", col = "black", main = "Norm Score Delta R2 in Validation", ylab = "Delta R2", xlab = "Number of terms", ylim=c(min(tab$delta.r2.test, na.rm = TRUE),max(tab$delta.r2.test, na.rm = TRUE)))
+    abline(h = 0, col = 3, lty = 2)
+  }
+  cat("The simulation yielded the following optimal settings:\n")
+  cat(paste0("\nNumber of terms with best crossfit: ", which.min((1-tab$crossfit)^2)))
+  cat(paste0("\nNumber of terms with best raw validation RMSE: ", which.min(tab$RMSE.raw.test)))
+  cat(paste0("\nNumber of terms with best norm validation R2: ", which.max(r2.test)))
+  cat("\nPlease investiate the plots and the summary table, as the results might vary within a narrow range.")
+  cat("\nEspacially pay attention to RMSE.raw.test, r2.test, crossfit near 1 and where delta R2 stops to progress.")
+  cat("\n")
+  return(tab)
+}
